/* vlab-cdb-lookuptest.c: It allows extract molecule record from a local chemical database.

    Copyright (C) 2001-2099, Rajkumar Buyya (rajkumar@buyya.com)!

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details @ www.gnu.org
*/

#include <sys/types.h>
#include <unistd.h> 
#include "header.h"

extern void *service_dispatch();
int main( int argc, char *argv[] )
{
   int clilen;
   struct sockaddr_in serv_addr;
  char pdb_filename[BUFFSIZE];
  long mol_no;

	printf("%s: Virtual Lab Chemical Database Lookup Test -- R. Buyya, 2001.\n", argv[0]);

	if( argc == 3 )
	{
		strcpy(pdb_filename, argv[1]);
		mol_no = atol(argv[2]);
	}
	else
	{
		printf( "Usage: %s <database_name> <molecule_number>\n", argv[0] );
		exit( 1 );
	}

	GetMoleculeRecordFromFile(pdb_filename, mol_no);
	return 0;
}

/* Lookup Table file is same as PDB file name with extension .idx */
void MakeLookupFileName( char *pdbfile, char *lookupfile )
{
   int i;
   for(i = 0; pdbfile[i]; i++)
   {
      if(pdbfile[i] == '.')
         break;
      lookupfile[i] = pdbfile[i];
   }
   lookupfile[i]='\0';
   strcat(lookupfile, ".idx");
}

int GetMoleculeRecordFromFile(char *pdb_filename, long mol_no)
{
	char MolFile[BUFFSIZE];
	char buff[BUFFSIZE];
	char pdb_idx_filename[BUFFSIZE];
	int pdbfd;
	int idxfd;
	int molfd;
	int len;
	int status1, status2;
	off_t record_location;
	int record_length;


	/* Check if Lookup Table for Database exist */
	MakeLookupFileName(pdb_filename, pdb_idx_filename);
	if( (idxfd = open( pdb_idx_filename, O_RDONLY|O_NONBLOCK )) == -1 )
	{
		printf("Error: Unable to open file--%s for reading\n", pdb_idx_filename);
      return 0;
   }
	lseek(idxfd, (mol_no-1)*(sizeof(off_t)+sizeof(int)), SEEK_SET );
	status1 = read( idxfd, &record_location, sizeof(off_t));
	status2 = read( idxfd, &record_length, sizeof(int));
	if( status1 == 0 || status2 == 0 )
	{
		printf("Record for Molecule #%d does not exist\n", mol_no );
		return 0;
	}
	close(idxfd);
	
	if( (pdbfd = open( pdb_filename, O_RDONLY|O_NONBLOCK )) == -1 )
	{
		printf("Error: Unable to open file--%s for reading\n", pdb_filename);
      return 0;
   }

	/* Open file for storing molecule info */
	sprintf( MolFile, "%ld.mol2", mol_no);
	if( (molfd = open( MolFile, O_CREAT | O_TRUNC | O_RDWR, S_IRWXU )) == -1 )
   {
      printf("Error: Unable to open file--%s for writting\n", MolFile);
      return 0;
   }

	printf("Record for Molecule #%d is found.\n", mol_no );
	printf("Location: %d, Length=%d\n", record_location, record_length );

	/* read molecule record and store into a separate file/transfer*/
	lseek(pdbfd, record_location, SEEK_SET );
	while(record_length > 0)
	{
		if( record_length < BUFFSIZE )
			len = read(pdbfd, buff, record_length);
		else
			len = read(pdbfd, buff, BUFFSIZE);
		write(molfd, buff, len );
		record_length -= BUFFSIZE;
	}
	close(pdbfd);
	close(molfd);
	return SUCCESS;
}
